@@ -7,7 +7,7 @@
  */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,13 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "times.h"
+#include "ginac.h"
+#include "timer.h"
+using namespace GiNaC;
+
+#include <iostream>
+#include <vector>
+using namespace std;
 
 static unsigned vandermonde_det(unsigned size)
 {
@@ -66,7 +72,6 @@ unsigned time_vandermonde()
 	unsigned result = 0;
 	
 	cout << "timing determinant of univariate symbolic Vandermonde matrices" << flush;
-	clog << "-------determinant of univariate symbolic Vandermonde matrices:" << endl;
 	
 	vector<unsigned> sizes;
 	vector<double> times;
@@ -90,12 +95,6 @@ unsigned time_vandermonde()
 		cout << '.' << flush;
 	}
 	
-	if (!result) {
-		cout << " passed ";
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed ";
-	}
 	// print the report:
 	cout << endl << "	dim:   ";
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i)
@@ -107,3 +106,12 @@ unsigned time_vandermonde()
 	
 	return result;
 }
+
+extern void randomify_symbol_serials();
+
+int main(int argc, char** argv)
+{
+	randomify_symbol_serials();
+	cout << setprecision(2) << showpoint;
+	return time_vandermonde();
+}
