@@ -3,7 +3,7 @@
  *  Interface to GiNaC's light-weight expression handles. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,19 +20,18 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_EX_H__
-#define __GINAC_EX_H__
-
-#include <iosfwd>
-#include <iterator>
-#include <functional>
-#include <stack>
+#ifndef GINAC_EX_H
+#define GINAC_EX_H
 
 #include "basic.h"
 #include "ptr.h"
 
-namespace GiNaC {
+#include <functional>
+#include <iosfwd>
+#include <iterator>
+#include <stack>
 
+namespace GiNaC {
 
 /** Helper class to initialize the library.  There must be one static object
  *  of this class in every object file that makes use of our flyweights in
@@ -43,6 +42,7 @@ namespace GiNaC {
  *
  *  @see utils.cpp */
 class library_init {
+	static void init_unarchivers();
 public:
 	library_init();
 	~library_init();
@@ -52,13 +52,6 @@ private:
 /** For construction of flyweights, etc. */
 static library_init library_initializer;
 
-/** Rotate bits of unsigned value by one bit to the left.
-  * This can be necesary if the user wants to define its own hashes. */
-inline unsigned rotate_left(unsigned n)
-{
-	return (n & 0x80000000U) ? (n << 1 | 0x00000001U) : (n << 1);
-}
-
 class scalar_products;
 class const_iterator;
 class const_preorder_iterator;
@@ -148,9 +141,9 @@ public:
 
 	// pattern matching
 	bool has(const ex & pattern, unsigned options = 0) const { return bp->has(pattern, options); }
-	bool find(const ex & pattern, lst & found) const;
+	bool find(const ex & pattern, exset& found) const;
 	bool match(const ex & pattern) const;
-	bool match(const ex & pattern, lst & repl_lst) const { return bp->match(pattern, repl_lst); }
+	bool match(const ex & pattern, exmap & repls) const { return bp->match(pattern, repls); }
 
 	// substitutions
 	ex subs(const exmap & m, unsigned options = 0) const;
@@ -224,7 +217,7 @@ public:
 
 	// noncommutativity
 	unsigned return_type() const { return bp->return_type(); }
-	tinfo_t return_type_tinfo() const { return bp->return_type_tinfo(); }
+	return_type_t return_type_tinfo() const { return bp->return_type_tinfo(); }
 
 	unsigned gethash() const { return bp->gethash(); }
 
@@ -709,7 +702,7 @@ inline ex imag_part(const ex & thisex)
 inline bool has(const ex & thisex, const ex & pattern, unsigned options = 0)
 { return thisex.has(pattern, options); }
 
-inline bool find(const ex & thisex, const ex & pattern, lst & found)
+inline bool find(const ex & thisex, const ex & pattern, exset& found)
 { return thisex.find(pattern, found); }
 
 inline bool is_polynomial(const ex & thisex, const ex & vars)
@@ -769,7 +762,7 @@ inline ex diff(const ex & thisex, const symbol & s, unsigned nth = 1)
 inline ex series(const ex & thisex, const ex & r, int order, unsigned options = 0)
 { return thisex.series(r, order, options); }
 
-inline bool match(const ex & thisex, const ex & pattern, lst & repl_lst)
+inline bool match(const ex & thisex, const ex & pattern, exmap& repl_lst)
 { return thisex.match(pattern, repl_lst); }
 
 inline ex simplify_indexed(const ex & thisex, unsigned options = 0)
@@ -982,4 +975,4 @@ inline void iter_swap(list<GiNaC::ex>::iterator i1, list<GiNaC::ex>::iterator i2
 
 } // namespace std
 
-#endif // ndef __GINAC_EX_H__
+#endif // ndef GINAC_EX_H
