@@ -3,7 +3,7 @@
  *  Implementation of abstract derivatives of functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-
 #include "fderivative.h"
 #include "operators.h"
 #include "archive.h"
 #include "utils.h"
 
+#include <iostream>
+
 namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(fderivative, function,
@@ -40,7 +40,6 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(fderivative, function,
 
 fderivative::fderivative()
 {
-	tinfo_key = &fderivative::tinfo_static;
 }
 
 //////////
@@ -50,25 +49,23 @@ fderivative::fderivative()
 fderivative::fderivative(unsigned ser, unsigned param, const exvector & args) : function(ser, args)
 {
 	parameter_set.insert(param);
-	tinfo_key = &fderivative::tinfo_static;
 }
 
 fderivative::fderivative(unsigned ser, const paramset & params, const exvector & args) : function(ser, args), parameter_set(params)
 {
-	tinfo_key = &fderivative::tinfo_static;
 }
 
 fderivative::fderivative(unsigned ser, const paramset & params, std::auto_ptr<exvector> vp) : function(ser, vp), parameter_set(params)
 {
-	tinfo_key = &fderivative::tinfo_static;
 }
 
 //////////
 // archiving
 //////////
 
-fderivative::fderivative(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
+void fderivative::read_archive(const archive_node& n, lst& sym_lst)
 {
+	inherited::read_archive(n, sym_lst);
 	unsigned i = 0;
 	while (true) {
 		unsigned u;
@@ -79,6 +76,7 @@ fderivative::fderivative(const archive_node &n, lst &sym_lst) : inherited(n, sym
 		++i;
 	}
 }
+GINAC_BIND_UNARCHIVER(fderivative);
 
 void fderivative::archive(archive_node &n) const
 {
@@ -90,7 +88,6 @@ void fderivative::archive(archive_node &n) const
 	}
 }
 
-DEFAULT_UNARCHIVE(fderivative)
 
 //////////
 // functions overriding virtual functions from base classes
