@@ -328,7 +328,7 @@ REGISTER_FUNCTION(beta, eval_func(beta_eval).
                         evalf_func(beta_evalf).
                         derivative_func(beta_deriv).
                         series_func(beta_series).
-                        latex_name("\\mbox{B}").
+                        latex_name("\\mathrm{B}").
 						set_symmetry(sy_symm(0, 1)));
 
 
