@@ -278,6 +278,16 @@ bool mul::info(unsigned inf) const
 		case info_flags::integer_polynomial:
 		case info_flags::cinteger_polynomial:
 		case info_flags::rational_polynomial:
+		case info_flags::real:
+		case info_flags::rational:
+		case info_flags::integer:
+		case info_flags::crational:
+		case info_flags::cinteger:
+		case info_flags::positive:
+		case info_flags::nonnegative:
+		case info_flags::posint:
+		case info_flags::nonnegint:
+		case info_flags::even:
 		case info_flags::crational_polynomial:
 		case info_flags::rational_function: {
 			epvector::const_iterator i = seq.begin(), end = seq.end();
@@ -286,6 +296,8 @@ bool mul::info(unsigned inf) const
 					return false;
 				++i;
 			}
+			if (overall_coeff.is_equal(*_num1_p) && inf == info_flags::even)
+				return true;
 			return overall_coeff.info(inf);
 		}
 		case info_flags::algebraic: {
