@@ -4,7 +4,7 @@
  *  in GiNaC functions */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,12 +21,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <stdexcept>
-
 #include "function.h"
 #include "utils.h"
 #include "remember.h"
 
+#include <stdexcept>
+
 namespace GiNaC {
 
 //////////
@@ -183,8 +183,8 @@ void remember_table::init_table()
 
 std::vector<remember_table> & remember_table::remember_tables()
 {
-	static std::vector<remember_table> * rt = new std::vector<remember_table>;
-	return *rt;
+	static std::vector<remember_table> rt = std::vector<remember_table>();
+	return rt;
 }
 
 } // namespace GiNaC
